%  Example to simulate Quaternion Least Mean Square algorithm to predict Lorenz
%  Attractor
%  C. Cheong Took and D. P. Mandic, The Quaternion LMS Algorithm for     
%  Adaptive Filtering of Hypercomplex Processes, IEEE Transactions on 
%  Signal Processing, vol. 57, no. 4, pp. 1316-1327, 2009;  
% 
%  INPUT:
%  x1: input signal 
%  l: filter length
%  mu: step-size
% 
%  OUTPUT:
%  y1: filter output
% 
% 
%  
%(c) Copyright  Clive Cheong Took 2013 (c.cheongtook@surrey.ac.uk).
%https://www.surrey.ac.uk/computing/research/nice/people/clive_cheong_took/
%  
%  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%     This program is free software; you can redistribute it and/or modify
%     it under the terms of the GNU General Public License as published by
%     the Free Software Foundation; either version 2 of the License, or
%     (at your option) any later version.
%  
%     This program is distributed in the hope that it will be useful,
%     but WITHOUT ANY WARRANTY; without even the implied warranty of
%     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%     GNU General Public License for more details.
%  
%     You can obtain a copy of the GNU General Public License from
%     http://www.gnu.org/copyleft/gpl.html or by writing to
%     Free Software Foundation, Inc.,675 Mass Ave, Cambridge, MA 02139, USA.
%  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% This implementation uses a "modified" version of the Quaternion toolbox   
% for Matlab available at http://qtfm.sourceforge.net/. The "modified"
% version is included in this package for the purpose of simulation on
% quaternion signal processing by Clive Cheong Took.
%  

clear all,close all

% Simulation of Lorenz Attractor
%http://en.wikipedia.org/wiki/Lorenz_system
time=25;
[t,x11] = ode45('func',[0 time],[1;2;3]);

L=length(x11); % length of data
l = 5; %Order of prediction filter
N=length(x11)-1 ; %For training	

x1=(quaternion(zeros(L,1),x11(:,1),x11(:,2),x11(:,3)));

d=x1;
y1 = transpose(quaternion(zeros(1,L),zeros(1,L),zeros(1,L),zeros(1,L)));
e=y1;
w = transpose(quaternion(zeros(1,l),zeros(1,l),zeros(1,l),zeros(1,l)));

%begin of QLMS algorithm
for n = l : N
	u = x1(n:-1:n-l+1) ;
    
    y1(n+1)= sum(times(w,u));
    
    
    e(n) = d(n+1) - y1(n+1) ;
% Start with big mu for speeding the convergence then slow down to reach the correct weights
%     if n < 20
%         mu=0.01;
%     else
%         mu=0.001;
%     end
mu=10^-6;    

 	w = w + mu *( 2*times(e(n),conj(u)) - times(conj(u),conj(e(n))) ) ;
    
end 

%check of results
for n =  N+1 : L-1
	u = x1(n:-1:n-l+1) ;
    y1(n+1)= sum(times(w,u));
    e(n) = d(n+1) - y1(n+1) ;
end 

start=1;
endd=length(x1);

figure(1)

subplot(311),plot(x(x1(start:endd)),'r')
hold on,plot(x(y1(start:endd)),'b-.'),ylabel('X')

subplot(312),plot(y(x1(start:endd)),'r')
hold on,plot(y(y1(start:endd)),'b-.'),ylabel('Y')

subplot(313),plot(z(x1(start:endd)),'r')
hold on,plot(z(y1(start:endd)),'b-.'),ylabel('Z')
legend('Actual','Estimated')



figure,plot3(x11(:,1),x11(:,2),x11(:,3),'r'),
hold on,plot3(x(y1(start:endd)),y(y1(start:endd)),z(y1(start:endd)),'b-.'),grid, xlabel('X'),ylabel('Y'),zlabel('Z')
legend('Actual','Estimated')
